﻿using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Engines;
using Org.BouncyCastle.Crypto.Modes;
using Org.BouncyCastle.Crypto.Paddings;
using Org.BouncyCastle.Crypto.Parameters;
using System.Text;

namespace IRaCIS.Core.Application.BusinessFilter;

public class AesEncryption
{
    // AES 加密（不带 IV）
    public static string Encrypt(string plainText, string key)
    {
        var keyBytes = Encoding.UTF8.GetBytes(key);

        // 使用 AES 引擎 + PKCS7 填充
        var engine = new AesEngine();
        var blockCipher = new PaddedBufferedBlockCipher(engine, new Pkcs7Padding());
        blockCipher.Init(true, new KeyParameter(keyBytes)); // true 表示加密

        var inputBytes = Encoding.UTF8.GetBytes(plainText);
        var encryptedBytes = ProcessCipher(blockCipher, inputBytes);

        // 返回 Base64 编码的加密字符串
        return Convert.ToBase64String(encryptedBytes);
    }

    // AES 解密（不带 IV）
    public static string Decrypt(string encryptedText, string key)
    {
        var keyBytes = Encoding.UTF8.GetBytes(key);
        var cipherBytes = Convert.FromBase64String(encryptedText);

        // 使用 AES 引擎 + PKCS7 填充
        var engine = new AesEngine();
        var blockCipher = new PaddedBufferedBlockCipher(engine, new Pkcs7Padding());
        blockCipher.Init(false, new KeyParameter(keyBytes)); // false 表示解密

        var decryptedBytes = ProcessCipher(blockCipher, cipherBytes);
        return Encoding.UTF8.GetString(decryptedBytes);
    }

    // AES 加密（带 IV）
    public static string Encrypt(string plainText, string key, string iv)
    {
        var keyBytes = Encoding.UTF8.GetBytes(key);
        var ivBytes = Encoding.UTF8.GetBytes(iv);

        // 使用 AES 引擎 + PKCS7 填充 + CBC 模式
        var engine = new AesEngine();
        var blockCipher = new PaddedBufferedBlockCipher(new CbcBlockCipher(engine), new Pkcs7Padding());
        blockCipher.Init(true, new ParametersWithIV(new KeyParameter(keyBytes), ivBytes)); // true 表示加密

        var inputBytes = Encoding.UTF8.GetBytes(plainText);
        var encryptedBytes = ProcessCipher(blockCipher, inputBytes);

        // 返回 Base64 编码的加密字符串
        return Convert.ToBase64String(encryptedBytes);
    }

    // AES 解密（带 IV）
    public static string Decrypt(string encryptedText, string key, string iv)
    {
        var keyBytes = Encoding.UTF8.GetBytes(key);
        var ivBytes = Encoding.UTF8.GetBytes(iv);
        var cipherBytes = Convert.FromBase64String(encryptedText);

        // 使用 AES 引擎 + PKCS7 填充 + CBC 模式
        var engine = new AesEngine();
        var blockCipher = new PaddedBufferedBlockCipher(new CbcBlockCipher(engine), new Pkcs7Padding());
        blockCipher.Init(false, new ParametersWithIV(new KeyParameter(keyBytes), ivBytes)); // false 表示解密

        var decryptedBytes = ProcessCipher(blockCipher, cipherBytes);
        return Encoding.UTF8.GetString(decryptedBytes);
    }

    // 处理加密/解密数据
    private static byte[] ProcessCipher(IBufferedCipher cipher, byte[] input)
    {
        var output = new byte[cipher.GetOutputSize(input.Length)];
        int length = cipher.ProcessBytes(input, 0, input.Length, output, 0);
        length += cipher.DoFinal(output, length);
        Array.Resize(ref output, length); // 调整输出数组大小以适应实际数据长度
        return output;
    }
}
